﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/// <summary>
/// Creates 2 LineRenderer according to Sin() function and animates it.
/// </summary>
public class LineController : MonoBehaviour {

	#region Fields
	public Color m_c1 = Color.yellow;
	public Color m_c2 = Color.red;
	GameObject m_ChildOne;
	GameObject m_ChildTwo;
	LineRenderer m_lineRendererOne;
	LineRenderer m_lineRendererTwo;

	public int m_lengthOfLineRenderer = 20;
	public LevelManager m_LevelManager;

	private Level m_CurrentLevel;

	private float m_CoMultiplier = 5f;

	public float m_HieghtMultiplier = 1f;
	public float m_WidthtMultiplier = 0.5f;
	public float m_Phase = 0.1f;
	public float m_XOffset = 1f;
	public float m_YOffset = 1f;
	float m_LineWidth = 0.1f;
	#endregion

	#region MonoBehaviour methods
	void Start() {

		m_ChildOne = new GameObject ();
		m_ChildOne.name = "Players Line";

		m_ChildTwo = new GameObject ();
		m_ChildTwo.name = "Target Line";
		m_ChildOne.transform.SetParent (gameObject.transform);
		m_ChildTwo.transform.SetParent (gameObject.transform);

		CreateRenderer (m_c1, m_ChildOne, ref m_lineRendererOne);
		CreateRenderer (m_c2, m_ChildTwo, ref m_lineRendererTwo);

	}


	void Update() {

		var t = Time.time;
		for(int i = 0; i < m_lengthOfLineRenderer; i++) {			
			m_lineRendererOne.SetPosition(i, new Vector3(m_XOffset +( i * m_WidthtMultiplier * m_CoMultiplier), (Mathf.Sin(i +m_Phase)*m_HieghtMultiplier * m_CoMultiplier) +m_YOffset, 0f));
			m_lineRendererTwo.SetPosition(i, new Vector3(m_XOffset +(i * m_CurrentLevel.Length * m_CoMultiplier), Mathf.Sin(i + t+ m_CurrentLevel.Phase) *m_CurrentLevel.Height * m_CoMultiplier + m_YOffset, 0.0f));	
		}


	}


	void OnEnable(){
		m_LevelManager.OnLevelAssigned += SetLevel;
		m_LevelManager.OnHightChanged += OnHeightChanged;
		m_LevelManager.OnLengthChanged += OnWidthChanged;
		m_LevelManager.OnPhaseChanged += OnPhaseChanged;
	}

	void OnDisable(){
		m_LevelManager.OnLevelAssigned -= SetLevel;
		m_LevelManager.OnHightChanged += OnHeightChanged;
		m_LevelManager.OnLengthChanged += OnWidthChanged;
		m_LevelManager.OnPhaseChanged += OnPhaseChanged;
	}
	#endregion

	#region Private methods

	void CreateRenderer(Color c, GameObject parent, ref LineRenderer l){

		l = parent.AddComponent<LineRenderer>();
		l.material = new Material (Shader.Find("Particles/Additive"));
		l.widthMultiplier = m_LineWidth;
		l.numPositions = m_lengthOfLineRenderer;
		l.startColor = c;
		l.endColor = c;
		l.gameObject.layer = LayerMask.NameToLayer ("Line");
	}

	void SetLevel(Level lvl){
		Debug.Log ("H = " + lvl.Height);
		Debug.Log ("L = " + lvl.Length);
		Debug.Log ("P = " + lvl.Phase);
		m_CurrentLevel = lvl;
	}

	#endregion

	#region Callbacks
	void OnHeightChanged(float h){
		m_HieghtMultiplier = h;
	}

	void OnWidthChanged(float w){
		m_WidthtMultiplier = w;
	}

	void OnPhaseChanged(float p){
		m_Phase = p;
	}

	#endregion

}
