﻿using UnityEngine;
using System;

public interface IInteractable
{
    void ChangeValue(float f);
    void ChangeState(Interactions.State state);
}

[Serializable]
public class Interactions
{
    public enum State
    {
        Inactive = 0,
        Waiting = 1,
        Active = 2,
        Collected = 3,
        Dropped = 4
    }

    public enum Type
    {
        Height = 0,
        Length = 1,
        Phase = 2,
        Energy = 3,
        None = 4
    }

    public float m_CurrentValue = 0f;
    public Type m_Type = Type.Height;
    public State m_State = State.Inactive;

    public void SetState(State state)
    {
        m_State = state;
    }

    public void SetType(Type type)
    {
        m_Type = type;
    }
}

public class Interactable : MonoBehaviour, IInteractable
{
    public Interactions m_Interactions;
    public Action<float, Interactions.Type> OnValueChanged;
    public Action<Interactions.State, Interactions.Type> OnStateChanged;

    public void ChangeValue(float value)
    {
        m_Interactions.m_CurrentValue = value;

        if (OnValueChanged != null)
        {
            OnValueChanged(m_Interactions.m_CurrentValue, m_Interactions.m_Type);
        }
    }

    public void ChangeState(Interactions.State state)
    {
        m_Interactions.m_State = state;

        if (OnStateChanged != null)
        {
            OnStateChanged(m_Interactions.m_State, m_Interactions.m_Type);
        }
    }
}