﻿using UnityEngine;

public class LengthOperatable : Operatable
{
    private float value = 0f;
    private Vector2[] hitSpots;
    private bool isClockwise;

    public LengthOperatable(Machine machine) : base(machine)
    {
        hitSpots = new Vector2[4];
        ResetHitSpots();
    }

    public override void OnUpdate()
    {

    }

    public override void OnXButtonDown()
    {
        if (machine.Interactable.m_Interactions.m_State == Interactions.State.Waiting)
        {
            machine.Interactable.ChangeState(Interactions.State.Active);
        }
        else if (machine.Interactable.m_Interactions.m_State == Interactions.State.Active)
        {
            machine.Interactable.ChangeState(Interactions.State.Waiting);
        }
    }

    public override void OnAButtonDown()
    {
        if (machine.Interactable.m_Interactions.m_State == Interactions.State.Active)
        {
            machine.Interactable.ChangeState(Interactions.State.Collected);
        }
    }

    public override void OnLeftStickValueChanged(Vector2 value)
    {
        if (Mathf.Approximately(value.magnitude, 1f))
        {
            Vector2 lastValue = Vector2.zero;
            bool isZeroFound = false;

            for (int i = 0; i < hitSpots.Length; i++)
            {
                if (hitSpots[i] == Vector2.zero)
                {
                    isZeroFound = true;
                    hitSpots[i] = value;
                    break;
                }

                lastValue = hitSpots[i];
            }

            if (isZeroFound == false)
            {
                ResetHitSpots();
                hitSpots[0] = value;
            }

            // Clockwise check.
            if ((lastValue == Vector2.left && value == Vector2.up)
                || (lastValue == Vector2.up && value == Vector2.right)
                || (lastValue == Vector2.right && value == Vector2.down)
                || (lastValue == Vector2.down && value == Vector2.left))
            {
                isClockwise = true;
            }

            // Counter-clockwise check.
            if ((lastValue == Vector2.left && value == Vector2.down)
                || (lastValue == Vector2.down && value == Vector2.right)
                || (lastValue == Vector2.right && value == Vector2.up)
                || (lastValue == Vector2.up && value == Vector2.left))
            {
                isClockwise = false;
            }

            // Set value
            if (isClockwise == true)
            {
                this.value += 0.01f;
            }
            else
            {
                this.value -= 0.01f;
            }

            if (this.value > 1f)
            {
                this.value = 1f;
            }

            if (this.value < 0f)
            {
                this.value = 0f;
            }

            machine.Interactable.ChangeValue(this.value);
        }
    }

    private void ResetHitSpots()
    {
        for (int i = 0; i < hitSpots.Length; i++)
        {
            hitSpots[i] = Vector2.zero;
        }
    }
}