﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;

public class InteractableUI : MonoBehaviour
{

    #region Fields
    [SerializeField]
    private Interactable m_Interactable;
    [SerializeField]
	private GameObject m_Container;
    [SerializeField]
    private Image m_MovingPart;

    float m_TweendDurationTime = 0.1f;
    private Tween m_Tween;
    #endregion


    #region MonoBehaviour methods
    //Test
    //	void Update(){
    //		if(Input.GetKeyDown(KeyCode.A)){
    //			Debug.Log ("KeyCode.A");
    //			OnValueChanged (Random.Range (0f, 1f), m_Interactable.m_Interactions.m_Type);
    //		}
    //
    //		if (Input.GetKeyDown (KeyCode.S)) {
    //			Debug.Log ("KeyCode.S");
    //			m_Interactable.m_Interactions.m_State = Interactions.State.Active;
    //			OnStateChanged (Interactions.State.Active, m_Interactable.m_Interactions.m_Type);
    //
    //		}
    //		if (Input.GetKeyDown (KeyCode.D)) {
    //			Debug.Log ("KeyCode.D");
    //			m_Interactable.m_Interactions.m_State = Interactions.State.Waiting;
    //			OnStateChanged (Interactions.State.Waiting, m_Interactable.m_Interactions.m_Type);
    //		}
    //	}

    void OnEnable()
    {
        if (m_Interactable == null)
        {
            m_Interactable = GetComponentInParent<Interactable>();
            OnStateChanged(m_Interactable.m_Interactions.m_State, m_Interactable.m_Interactions.m_Type);
        }
        m_Interactable.OnValueChanged += OnValueChanged;
        m_Interactable.OnStateChanged += OnStateChanged;
    }

    void OnDisable()
    {
        m_Interactable.OnValueChanged -= OnValueChanged;
        m_Interactable.OnStateChanged -= OnStateChanged;
    }
    #endregion

    #region Callbacks
    void OnValueChanged(float f, Interactions.Type t)
    {
        if (m_Interactable.m_Interactions.m_State == Interactions.State.Active)
        {


            switch (t)
            {
                case Interactions.Type.Energy:
                    m_Tween = m_MovingPart.DOFillAmount(f, m_TweendDurationTime);
                    break;
                case Interactions.Type.Height:
                    m_Tween = m_MovingPart.DOFillAmount(f, m_TweendDurationTime);
                    break;
                case Interactions.Type.Length:
                    m_Tween = m_MovingPart.transform.DORotate(new Vector3(0f, 0f, f * 360f), m_TweendDurationTime);
                    break;
                case Interactions.Type.Phase:
                    m_Tween = m_MovingPart.transform.DORotate(new Vector3(0f, 0f, (f * -180f) + 90f), m_TweendDurationTime);
                    break;
                case Interactions.Type.None:
                    break;
            }
        }
    }

    void OnStateChanged(Interactions.State s, Interactions.Type t)
    {
        switch (s)
        {
            case Interactions.State.Active:
                m_Container.SetActive(true);
                break;
            case Interactions.State.Waiting:
                m_Tween.Kill(false);
                m_Container.SetActive(false);
                break;
            case Interactions.State.Collected:
                m_Tween.Kill(false);
                m_Container.SetActive(false);

                Destroy(gameObject);
                break;
            case Interactions.State.Inactive:
                //Buraya eğer gri tonda bir şey gelecekse eklenebilir
                break;
        }

    }
    #endregion

    #region Tweens

    void ChangeEnergyValue(float f)
    {

    }
    #endregion
}
